#pragma once

#include "CoreMinimal.h"
#include "Walk.h"
#include "CityMap.h"
#include "CitySettings.h"
#include "BuildingTile.h"
#include "FoodTile.h"

enum EAgentState {
	Home,
	Searching,
	Waiting,
	WalkingAround,
	Eating,
	Returning,
	GoingToHomeDoor
};

/**
 * Represents an agent participating in the simulation - a single hungry human.
 */
class CV10_DAV_API FAgent
{
public:
	FAgent(FBuildingTile* Home, FVector Offset, FAgentSettings& Settings);
	~FAgent();

	void ForcesCompute(FCityMap* Map, TArray<FAgent*>& Others);
	void ForcesApply(float DeltaTime);

	FVector GetPosition() const;

	float GetFeatureState() const;

	FTransform Transform;

private:
	void UpdateStatus(FCityMap* Map, TArray<FAgent*>& Others);

	bool MoveToStreet(FCityMap* Map);
	bool GoSearchForFood(FCityMap* Map);
	bool GoToFoodmarketDoor(FCityMap* Map);
	bool TryGetInsideFoodmarket(FCityMap* Map);
	bool WaitAtFoodmarketDoor(FCityMap* Map);
	bool IsGoingToWait(FCityMap* Map);
	bool NewMarketDiscovered(FCityMap* Map, TArray<FAgent*>& Others);
	bool PlanRandomWalk(FCityMap* Map);
	bool StartEating(FCityMap* Map);
	bool Eat(FCityMap* Map);
	bool GoHome(FCityMap* Map);
	bool GoToHomeDoor(FCityMap* Map);
	bool StayHome(FCityMap* Map);

	void CheckStoppingRule();
	void UpdateWaitInterval();

	float Dist(FAgent* Agent) const;
	int IsNotLookingForOthers() const;
	bool IsStopped() const;

	FVector OtherAgentsRepulsionForce(TArray<FAgent*>& Others) const;
	FVector OtherAgentsAvoidanceForce(TArray<FAgent*>& Others) const;

	//forces base
	FVector Position;
	FVector Velocity;
	FVector Acceleration;

	//display
	FVector WorldOffset;

	//forces
	FVector AttractorForce;
	FVector Steering;
	FVector AvoidanceWallForce;
	FVector AvoidanceAgentsForce;
	FVector Repulsion;
	FVector RepulsionWallForce;
	FVector RepulsionAgentsForce;

	//characteristics
	float Size;
	float Radius;
	float PersonalSpaceEpsilon;
	float PushingLambda;
	float WalkTolerance;
	float HungerLevel;

	float FoodCapacity;
	float FoodHungerDelta;
	float FoodEatingDelta;
	float PatienceLimit;
	float PatienceDelta;
	float TalkingDistance;

	float WeightWallRepulsion;
	float WeightAgentRepulsion;
	float WeightWallAvoidance;
	float WeightAgentAvoidance;
	float WeightSteeringForce;

	//stats
	float Food;
	float Patience;
	int StopCounter;
	int Alpha;
	EAgentState Status;

	FWalk WalkAction;
	FBuildingTile* HomeTile;
	FFoodTile* FoodTarget;
};