// Fill out your copyright notice in the Description page of Project Settings.


#include "BuildingTile.h"


FBuildingTile::FBuildingTile(int X, int Y, int W, int H, int Z)
	: FTile(X, Y, W, H, Z) {}

int FBuildingTile::GenerateMesh(TArray<FVector>& Vertices, TArray<FVector>& Normals, TArray<int32>& Indices, TArray<FVector2D>& UVs, const int IndexOffset, TArray<FLinearColor>& Color)
{
	int Count = GenerateBox(Vertices, Normals, Indices, UVs, IndexOffset, *this);

	for (int i = 0; i < Count; ++i)
		Color.Add(FLinearColor(0.5, 0.3, 0.1, 1.0));

	return Count;
}

FVector FBuildingTile::NearestWallNormal(const FVector& Pos) const
{
	float Dx = std::min(Pos.X - PosX, 0.0) + std::max(0.0, Pos.X - (PosX + W));
	float Dy = std::min(Pos.Y - PosY, 0.0) + std::max(0.0, Pos.Y - (PosY + H));

	//report normal only for 4-neighborhood
	if (Dx != 0 && Dy != 0)
		return FVector(0);

	return FVector(Dx, Dy, 0).GetSafeNormal();
}

int FBuildingTile::GetTileType()
{
	return ETileType::Building;
}


FVector FBuildingTile::DoorLocation(FTile* Street)
{
	FVector StreetLoc = Street->GetLocation();
	FVector InsideLoc = GetLocation();
	InsideLoc.Z = StreetLoc.Z;

	FVector Dir = InsideLoc - StreetLoc;
	Dir = Dir.GetClampedToMaxSize(std::min(W, H) / 2.0) * 0.6;

	return StreetLoc + Dir;
}
