#pragma once

#include "CoreMinimal.h"

#include "Agent.h"
#include "CityMap.h"

#include "ProceduralMeshComponent.h"
#include "GameFramework/Actor.h"
#include "CityGrid.generated.h"

/**
 *	The UE actor holding the simulation. Does the actual simulation tick, construction of the simulation and passing
 *	simulation parameters between the UE object/editor and the simulation classes. Also handles rendering using a
 *	procedural mesh.
 */
UCLASS()
class CV10_DAV_API ACityGrid : public AActor
{
	GENERATED_BODY()
	
public:	
	// Sets default values for this actor's properties
	ACityGrid();

	// Called every frame
	virtual void Tick(float DeltaTime) override;

	////// City Layout
	UPROPERTY(EditAnywhere, AssetRegistrySearchable, Category = "CityLayout")
		UTexture2D* Layout;
	UPROPERTY(EditAnywhere, Category = "CityLayout")
		bool GenerateMesh;
	UPROPERTY(EditAnywhere, Category = "CityLayout", meta = (ClampMin = "15.0", ClampMax = "40.0", UIMin = "15.0", UIMax = "40.0"))
		float Spacing;
	UPROPERTY(EditAnywhere, Category = "CityLayout", meta = (ClampMin = "15.0", ClampMax = "40.0", UIMin = "15.0", UIMax = "40.0"))
		float Height;
	UPROPERTY(EditAnywhere, Category = "Animation", meta = (ClampMin = "1.0", ClampMax = "20.0", UIMin = "1.0", UIMax = "20.0"))
		float Speed;
	UPROPERTY(EditAnywhere, Category = "FoodMarkets", meta = (ClampMin = "0.0", UIMin = "0.0"))
		int DisplayCustomersInRow;
	UPROPERTY(EditAnywhere, Category = "FoodMarkets", meta = (ClampMin = "0.0", UIMin = "0.0"))
		int LimitCustomersInside;

	////// Agents individual features
	UPROPERTY(EditAnywhere, Category = "Agents Individual", meta = (ClampMin = "0.0", ClampMax = "100.0", UIMin = "0.0", UIMax = "10.0"))
		float PersonalSpaceRangeMin;
	UPROPERTY(EditAnywhere, Category = "Agents Individual", meta = (ClampMin = "0.0", ClampMax = "100.0", UIMin = "0.0", UIMax = "10.0"))
		float PersonalSpaceRangeMax;

	UPROPERTY(EditAnywhere, Category = "Agents Individual", meta = (ClampMin = "0.0", ClampMax = "10.0", UIMin = "0.0", UIMax = "10.0"))
		float PushFactorRangeMin;
	UPROPERTY(EditAnywhere, Category = "Agents Individual", meta = (ClampMin = "0.0", ClampMax = "10.0", UIMin = "0.0", UIMax = "10.0"))
		float PushFactorRangeMax;

	UPROPERTY(EditAnywhere, Category = "Agents Individual", meta = (ClampMin = "0.1", ClampMax = "2.0", UIMin = "0.1", UIMax = "2.0"))
		float FoodHungerDeltaMin;
	UPROPERTY(EditAnywhere, Category = "Agents Individual", meta = (ClampMin = "0.1", ClampMax = "2.0", UIMin = "0.1", UIMax = "2.0"))
		float FoodHungerDeltaMax;

	UPROPERTY(EditAnywhere, Category = "Agents Individual", meta = (ClampMin = "0.1", ClampMax = "2.0", UIMin = "0.1", UIMax = "2.0"))
		float FoodEatingDeltaMin;
	UPROPERTY(EditAnywhere, Category = "Agents Individual", meta = (ClampMin = "0.1", ClampMax = "2.0", UIMin = "0.1", UIMax = "2.0"))
		float FoodEatingDeltaMax;

	UPROPERTY(EditAnywhere, Category = "Agents Individual", meta = (ClampMin = "10.0", ClampMax = "1000.0", UIMin = "10.0", UIMax = "1000.0"))
		float FoodCapacityMin;
	UPROPERTY(EditAnywhere, Category = "Agents Individual", meta = (ClampMin = "10.0", ClampMax = "1000.0", UIMin = "10.0", UIMax = "1000.0"))
		float FoodCapacityMax;

	UPROPERTY(EditAnywhere, Category = "Agents Individual", meta = (ClampMin = "10.0", ClampMax = "1000.0", UIMin = "10.0", UIMax = "1000.0"))
		float PatienceLimitMin;
	UPROPERTY(EditAnywhere, Category = "Agents Individual", meta = (ClampMin = "10.0", ClampMax = "1000.0", UIMin = "10.0", UIMax = "1000.0"))
		float PatienceLimitMax;

	UPROPERTY(EditAnywhere, Category = "Agents Individual", meta = (ClampMin = "0.1", ClampMax = "2.0", UIMin = "0.1", UIMax = "2.0"))
		float PatienceDeltaMin;
	UPROPERTY(EditAnywhere, Category = "Agents Individual", meta = (ClampMin = "0.1", ClampMax = "2.0", UIMin = "0.1", UIMax = "2.0"))
		float PatienceDeltaMax;

	//////Agents common features
	UPROPERTY(EditAnywhere, Category = "Agents Common", meta = (ClampMin = "1.0", ClampMax = "100.0", UIMin = "1.0", UIMax = "100.0"))
		float PerceptionRadius;
	UPROPERTY(EditAnywhere, Category = "Agents Common", meta = (ClampMin = "0.0", ClampMax = "20.0", UIMin = "0.0", UIMax = "10.0"))
		int WalkCheckpointTolerance;
	UPROPERTY(EditAnywhere, Category = "Agents Common", meta = (ClampMin = "0.0", ClampMax = "20.0", UIMin = "0.0", UIMax = "10.0"))
		int AgentRadius;
	UPROPERTY(EditAnywhere, Category = "Agents Common", meta = (ClampMin = "1.0", ClampMax = "10.0", UIMin = "1.0", UIMax = "10.0"))
		float TalkingDistance;
	UPROPERTY(EditAnywhere, Category = "Agents Common", meta = (ClampMin = "0.1", ClampMax = "10.0", UIMin = "0.1", UIMax = "10.0"))
		float WeightWallRepulsion;
	UPROPERTY(EditAnywhere, Category = "Agents Common", meta = (ClampMin = "0.1", ClampMax = "10.0", UIMin = "0.1", UIMax = "10.0"))
		float WeightAgentRepulsion;
	UPROPERTY(EditAnywhere, Category = "Agents Common", meta = (ClampMin = "0.1", ClampMax = "10.0", UIMin = "0.1", UIMax = "10.0"))
		float WeightWallAvoidance;
	UPROPERTY(EditAnywhere, Category = "Agents Common", meta = (ClampMin = "0.1", ClampMax = "10.0", UIMin = "0.1", UIMax = "10.0"))
		float WeightAgentAvoidance;
	UPROPERTY(EditAnywhere, Category = "Agents Common", meta = (ClampMin = "0.1", ClampMax = "100.0", UIMin = "0.1", UIMax = "100.0"))
		float WeightSteeringForce;
	UPROPERTY(EditAnywhere, Category = "Agents Common", meta = (ClampMin = "0.1", ClampMax = "100.0", UIMin = "0.1", UIMax = "100.0"))
		float HungerLevel;

	// Creates mesh
	virtual void OnConstruction(const FTransform& Transform) override;

protected:
	// Called when the game starts or when spawned
	virtual void BeginPlay() override;

	// Init agents and city
	void InitCity();

	// Init agents
	void InitAgents();
	
	// City grid mesh
	UPROPERTY()
	UProceduralMeshComponent* Mesh;
	UPROPERTY()
	UMaterial* Material;
	
	// City grid container
	FCityMap* Map;

	// Instanced agent meshes
	UPROPERTY()
	UInstancedStaticMeshComponent* AgentMesh;
	
	// Simulation agents
	TArray<FAgent*> Agents;
};
