#pragma once
#include <random>

#include "CoreMinimal.h"
#include "Tile.h"
#include "CitySettings.h"

/**
 *	Represents the whole map of a city. Holds all the tiles and can calculate forces for wall collision avoidance.
 */
class CV10_DAV_API FCityMap
{
public:
	FCityMap();
	~FCityMap();

	//generate city tiles and mesh according to supplied texture
	void Generate(const FCitySettings& Settings);

	const TArray<FTile*>& GetBuildings() const;
	
	FTile* GetTileForLocation(const FVector Loc) const;
	FTile* GetRandomStreetTile();
	FTile* GetNearestTile(FTile* Tile, ETileType Type) const;

	void EmptyOccupants();
	void GetOccupantsForPosition(TArray<FAgent*>& Occupants, const FVector& Pos, float Radius);

	FVector WallsAvoidanceForce(const FVector& Pos, const FVector& Dir, const float AgentRadius, const float Epsilon);
	FVector WallsRepulsionForce(const FVector& Pos, const float Radius, const float Epsilon);

	int GetRandomWaitInterval();
	
private:
	void GenerateCity(const FCitySettings& Settings);
	void GenerateMesh(const FCitySettings& Settings);

	int32 ResolutionX;
	int32 ResolutionY;
	int32 Res;
	int32 Spacing;
	int32 Height;
		
	TArray<FTile*> Tiles;
	TArray<FTile*> Buildings;
	TArray<FTile*> Food;
	TArray<FTile*> Streets;

	std::default_random_engine Generator;
	std::uniform_int_distribution<int> RandomStreet;
	std::uniform_int_distribution<int> RandomWaitInterval;
};
