#include "FoodTile.h"

#include "Tile.h"

FFoodTile::FFoodTile(int X, int Y, int W, int H, int Z, int AgentsLimit, int AgentsInRow)
	: FBuildingTile(X, Y, W, H, Z),
	CustomersLimit(AgentsLimit), CustomersPresent(0), DisplayAgentRow(AgentsInRow)
{
	for (int i = 0; i < CustomersLimit; ++i)
		Customers.Push(nullptr);
}

int FFoodTile::GenerateMesh(TArray<FVector>& Vertices, TArray<FVector>& Normals, TArray<int32>& Indices, TArray<FVector2D>& UVs, const int IndexOffset, TArray<FLinearColor>& Color)
{
	int Count = GenerateBox(Vertices, Normals, Indices, UVs, IndexOffset, *this);

	for (int i = 0; i < Count; ++i)
		Color.Add(FLinearColor(0.0, 0.45, 0.11, 1.0));

	return Count;
}

int FFoodTile::GetTileType()
{
	return ETileType::Food;
}

bool FFoodTile::TryAgentGetInside()
{
	if (CustomersLimit > CustomersPresent)
	{
		CustomersPresent++;
		return true;
	}

	return false;
}
void FFoodTile::AgentGetOut(const FAgent* Agent)
{
	CustomersPresent--;

	for (int i = 0; i < CustomersLimit; ++i)
		if (Customers[i] == Agent)
		{
			Customers[i] = nullptr;
			break;
		}
}

FVector FFoodTile::GetInsideAgentPosition(const FAgent* Agent)
{
	FVector Pos;
	int32 A, B, C;
	const float Border = 2;

	int Idx = 0;

	for (int i = 0; i < CustomersLimit; ++i)
		if (Customers[i] == nullptr)
		{
			Idx = i;
			Customers[i] = Agent;
			break;
		}

	A = Idx % DisplayAgentRow;
	B = (Idx / DisplayAgentRow) % DisplayAgentRow;
	C = Idx / (DisplayAgentRow * DisplayAgentRow);

	Pos = FVector(PosX, PosY, Z);

	float SpaceBetweenX = (((float)(W - 2 * Border)) / static_cast<float>(DisplayAgentRow));
	float SpaceBetweenY = (((float)(H - 2 * Border)) / static_cast<float>(DisplayAgentRow));

	Pos += FVector(Border + (A + 0.5) * SpaceBetweenX,
		Border + (B + 0.5) * SpaceBetweenY,
		W / 10 + C * SpaceBetweenX);

	return Pos;
}