#include "Tile.h"

FTile::FTile(int X, int Y, int W, int H, int Z)
	:X(X), Y(Y), PosX(X * W), PosY(Y * H), W(W), H(H), Z(Z) {}


FVector FTile::GetLocation() const
{
	return FVector(PosX + W / 2, PosY + H / 2, Z + W / 10);
}

const TArray<FTile*>& FTile::GetNeighbors() const
{
	return Neighbors;
}

void FTile::EmptyOccupants()
{
	Occupants.Empty();
}

void FTile::AddOccupant(FAgent* Agent)
{
	Occupants.Push(Agent);
}

void FTile::GetOccupants(TArray<FAgent*>& Agents) const
{
	for (auto o : Occupants)
		Agents.Push(o);
}

FVector FTile::GetVecToClosestTilePoint(const FVector& Pos)
{
	float Dx = std::min(Pos.X - PosX, 0.0) + std::max(0.0, Pos.X - (PosX + W));
	float Dy = std::min(Pos.Y - PosY, 0.0) + std::max(0.0, Pos.Y - (PosY + H));

	return FVector(Dx, Dy, 0);
}

FVector FTile::NearestWallNormal(const FVector& Pos) const
{
	return FVector(0);
}

void FTile::SetupNeighbors(TArray<FTile*> Tiles, int32 ResolutionX, int32 ResolutionY)
{
	//global index
	int32 Idx = Y * ResolutionX + X;
	int32 Nidx = Idx - ResolutionX; //init to top

	bool bTop = (Y == 0);
	bool bRight = (X == (ResolutionX - 1));
	bool bBottom = (Y == (ResolutionY - 1));
	bool bLeft = (X == 0);

	//line above
	if (!bTop)
		Neighbors.Push(Tiles[Nidx]);

	//current line
	Nidx += ResolutionX;
	if (!bLeft)
		Neighbors.Push(Tiles[Nidx - 1]);

	if (!bRight)
		Neighbors.Push(Tiles[Nidx + 1]);

	//line bellow
	Nidx += ResolutionX;
	if (!bBottom)
		Neighbors.Push(Tiles[Nidx]);
}

int FTile::GenerateBox(TArray<FVector>& Vertices, TArray<FVector>& Normals, TArray<int32>& Indices, TArray<FVector2D>& UVs, const int IndexOffset, FTile& Tile)
{
	FVector V[8] = {
	FVector(Tile.PosX, Tile.PosY, 0),
	FVector(Tile.PosX + Tile.W, Tile.PosY, 0),
	FVector(Tile.PosX + Tile.W, Tile.PosY + Tile.H, 0),
	FVector(Tile.PosX, Tile.PosY + Tile.H, 0),
	FVector(Tile.PosX, Tile.PosY, Tile.Z),
	FVector(Tile.PosX, Tile.PosY + Tile.H, Tile.Z),
	FVector(Tile.PosX + Tile.W, Tile.PosY + Tile.H, Tile.Z),
	FVector(Tile.PosX + Tile.W, Tile.PosY, Tile.Z)
	};

	FVector N[6] = {
		FVector(0, 0, 1),
		FVector(0, 0, -1),
		FVector(0, 1, 0),
		FVector(0, -1, 0),
		FVector(0, 0, 0),
		FVector(-1, 0, 0)
	};

	FVector2D U[4] = {
		FVector2D(0, 0),
		FVector2D(1, 0),
		FVector2D(1, 1),
		FVector2D(0, 1)
	};

	//bottom side
	Vertices.Add(V[0]);
	Vertices.Add(V[1]);
	Vertices.Add(V[2]);
	Vertices.Add(V[3]);
	Normals.Add(N[1]);
	Normals.Add(N[1]);
	Normals.Add(N[1]);
	Normals.Add(N[1]);
	UVs.Add(U[0]);
	UVs.Add(U[1]);
	UVs.Add(U[2]);
	UVs.Add(U[3]);

	//top side
	Vertices.Add(V[4]);
	Vertices.Add(V[5]);
	Vertices.Add(V[6]);
	Vertices.Add(V[7]);
	Normals.Add(N[0]);
	Normals.Add(N[0]);
	Normals.Add(N[0]);
	Normals.Add(N[0]);
	UVs.Add(U[0]);
	UVs.Add(U[1]);
	UVs.Add(U[2]);
	UVs.Add(U[3]);

	//front side
	Vertices.Add(V[0]);
	Vertices.Add(V[3]);
	Vertices.Add(V[5]);
	Vertices.Add(V[4]);
	Normals.Add(N[5]);
	Normals.Add(N[5]);
	Normals.Add(N[5]);
	Normals.Add(N[5]);
	UVs.Add(U[0]);
	UVs.Add(U[1]);
	UVs.Add(U[2]);
	UVs.Add(U[3]);

	//back side
	Vertices.Add(V[2]);
	Vertices.Add(V[1]);
	Vertices.Add(V[7]);
	Vertices.Add(V[6]);
	Normals.Add(N[4]);
	Normals.Add(N[4]);
	Normals.Add(N[4]);
	Normals.Add(N[4]);
	UVs.Add(U[0]);
	UVs.Add(U[1]);
	UVs.Add(U[2]);
	UVs.Add(U[3]);

	//left side
	Vertices.Add(V[1]);
	Vertices.Add(V[0]);
	Vertices.Add(V[4]);
	Vertices.Add(V[7]);
	Normals.Add(N[3]);
	Normals.Add(N[3]);
	Normals.Add(N[3]);
	Normals.Add(N[3]);
	UVs.Add(U[0]);
	UVs.Add(U[1]);
	UVs.Add(U[2]);
	UVs.Add(U[3]);

	//right side
	Vertices.Add(V[3]);
	Vertices.Add(V[2]);
	Vertices.Add(V[6]);
	Vertices.Add(V[5]);
	Normals.Add(N[2]);
	Normals.Add(N[2]);
	Normals.Add(N[2]);
	Normals.Add(N[2]);
	UVs.Add(U[0]);
	UVs.Add(U[1]);
	UVs.Add(U[2]);
	UVs.Add(U[3]);

	for (int side = 0; side < 6; side++)
	{
		Indices.Add(0 + IndexOffset + side * 4);
		Indices.Add(1 + IndexOffset + side * 4);
		Indices.Add(2 + IndexOffset + side * 4);
		Indices.Add(0 + IndexOffset + side * 4);
		Indices.Add(2 + IndexOffset + side * 4);
		Indices.Add(3 + IndexOffset + side * 4);
	}

	return 6 * 4;
}