#pragma once

#include "CoreMinimal.h"

class CV10_DAV_API FAgent;

enum ETileType {
	Border,
	Building,
	Street,
	Food
};

/**
 *	Represents a single city tile on the simulation grid.
 */
class CV10_DAV_API FTile
{
public:
	FTile(int X, int Y, int W, int H, int Z);
	virtual ~FTile() {};

	FVector GetLocation() const;

	//returns number of vertices generated
	virtual int GenerateMesh(TArray<FVector>& Vertices, TArray<FVector>& Normals, TArray<int32>& Indices, TArray<FVector2D>& UVs, const int IndexOffset, TArray<FLinearColor>& Color) = 0;

	virtual FVector NearestWallNormal(const FVector& Pos) const;
	const TArray<FTile*>& GetNeighbors() const;

	void SetupNeighbors(TArray<FTile*> Tiles, int32 ResolutionX, int32 ResolutionY);

	void EmptyOccupants();
	void AddOccupant(FAgent* Agent);
	void GetOccupants(TArray<FAgent*>& Agents) const;

	FVector GetVecToClosestTilePoint(const FVector& Pos);

	virtual int GetTileType() = 0;

protected:
	static int GenerateBox(TArray<FVector>& Vertices, TArray<FVector>& Normals, TArray<int32>& Indices, TArray<FVector2D>& UVs, const int IndexOffset, FTile& Tile);

	int X, Y, PosX, PosY, W, H, Z;

	TArray<FTile*> Neighbors;
	TArray<FAgent*> Occupants;
};
